# No Sync Scripts

> Prevent synchronous scripts.

### Why This Error Occurred

A synchronous script was used which can impact your webpage performance.

### Possible Ways to Fix It

#### Script component (recommended)

```jsx
import Script from 'next/script'

function Home() {
  return (
    <div class="container">
      <Script src="https://third-party-script.js"></Script>
      <div>Home Page</div>
    </div>
  )
}

export default Home
```

#### Use `async` or `defer`

```html
<script src="https://third-party-script.js" async />
<script src="https://third-party-script.js" defer />
```

### Useful Links

- [Efficiently load third-party JavaScript](https://web.dev/efficiently-load-third-party-javascript/)
